package com.ahazou.photo_editor_sdk

import android.app.Activity
import android.app.ProgressDialog
import android.content.Intent
import android.graphics.Bitmap
import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import ly.img.android.pesdk.PhotoEditorSettingsList
import ly.img.android.pesdk.assets.font.basic.FontPackBasic
import android.net.Uri
import android.util.Log
import com.ahazou.photo_editor_sdk.service.ApiClient
import com.ahazou.photo_editor_sdk.service.ApiService
import com.ahazou.photo_editor_sdk.service.LogoToResponse
import com.bumptech.glide.Glide
import com.bumptech.glide.load.DataSource
import com.bumptech.glide.load.engine.GlideException
import com.bumptech.glide.request.RequestListener
import com.bumptech.glide.request.target.Target
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.PluginRegistry
import ly.img.android.pesdk.backend.decoder.ImageSource
import ly.img.android.pesdk.backend.model.config.ImageStickerAsset
import ly.img.android.pesdk.backend.model.constant.Directory.DCIM
import ly.img.android.pesdk.backend.model.state.AssetConfig
import ly.img.android.pesdk.backend.model.state.LoadSettings
import ly.img.android.pesdk.backend.model.state.SaveSettings
import ly.img.android.pesdk.backend.model.state.manager.SettingsList
import ly.img.android.pesdk.ui.activity.ImgLyIntent
import ly.img.android.pesdk.ui.activity.PhotoEditorBuilder
import ly.img.android.pesdk.ui.model.state.*
import ly.img.android.pesdk.ui.panels.item.*
import okhttp3.MediaType
import okhttp3.MultipartBody
import okhttp3.RequestBody
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import java.io.File
import java.io.FileOutputStream
import java.lang.ref.WeakReference
import java.util.ArrayList

/** PhotoEditorSdkPlugin */
class PhotoEditorSdkPlugin:
        ActivityAware,
        FlutterPlugin,
        MethodCallHandler,
        PluginRegistry.ActivityResultListener {

  companion object {
    lateinit var activity: WeakReference<Activity>
    lateinit var arrLogos: ArrayList<CustomImageStickerAsset>
    lateinit var arrStickers: ArrayList<CustomImageStickerAsset>
    lateinit var userId: String
    lateinit var resultClass: Result
    lateinit var progressDialog: ProgressDialog
  }

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    val channel = MethodChannel(flutterPluginBinding.binaryMessenger, "photo_editor_sdk")
    channel.setMethodCallHandler(PhotoEditorSdkPlugin())
  }

  override fun onDetachedFromActivity() {
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activity = WeakReference(binding.activity)
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
  }

  override fun onDetachedFromActivityForConfigChanges() {
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    resultClass = result
    when ( call.method ) {
      "getPlatformVersion" -> result.success("Android ${android.os.Build.VERSION.RELEASE}")
      "addAllContents" -> {
        val args = call.arguments as Map<*, *>
        val logos: ArrayList<*> = args["logos"] as ArrayList<*>
        val stickers: ArrayList<*> = args["stickers"] as ArrayList<*>
        arrLogos = retrieveAssets(logos)
        arrStickers = retrieveAssets(stickers)
      }
      "editImageByUrl" -> {
        val args = call.arguments as? String
        val uri = Uri.parse(args)
        openEditor(uri)
      }
      "editImageByFile" -> {
        val args = call.arguments as? String
        val file = File(args)
        val uri = Uri.fromFile(file)
        openEditor(uri)
      }
      "setUserId" -> {
        val id = call.arguments as String
        userId = id
      }
      else -> result.notImplemented()
    }
  }

  private fun retrieveAssets(list: ArrayList<*>): ArrayList<CustomImageStickerAsset> {
    val result = ArrayList<CustomImageStickerAsset>()
    list.forEach {
      Glide
              .with(activity.get()!!)
              .asFile()
              .load(it)
              .listener(object : RequestListener<File> {
                override fun onLoadFailed(e: GlideException?, model: Any?,
                                          target: Target<File>?, isFirstResource: Boolean): Boolean {
                  return false
                }

                override fun onResourceReady(resource: File?, model: Any?, target: Target<File>?,
                                             dataSource: DataSource?, isFirstResource: Boolean): Boolean {
                  val uri = Uri.fromFile(resource)
                  val imageStickerConfig =
                          ImageStickerAsset.createTemporaryStickerAsset(uri)
                  result.add(CustomImageStickerAsset(imageStickerConfig, uri))
                  return false
                }
              })
              .submit()
    }
    return result
  }

  private fun createPesdkSettingsList(inputImage: Uri?) =
          PhotoEditorSettingsList()
                  .configure<UiConfigText> {
                    it.setFontList(FontPackBasic.getFontPack())
                  }
                  .configure<SaveSettings> {
                    it.setExportDir(DCIM, "AhazouPro")
                    it.setExportPrefix("result_")
                    it.savePolicy = SaveSettings.SavePolicy.RETURN_ALWAYS_ONLY_OUTPUT
                  }
                  .configure<LoadSettings> {
                    it.source = inputImage
                    it.isDeleteProtectedSource = true
                  }
                  .configure<UiConfigTheme> {
                    it.theme =  R.style.Imgly_Theme_TopActionBar_NoFullscreen
                  }
                  .configure<UiConfigMainMenu> {
                    it.setToolList(
                            ToolItem("imgly_tool_sticker_selection",
                                    R.string.pesdk_sticker_title_name,
                                    ImageSource.create(R.drawable.imgly_icon_tool_sticker)),
                            ToolItem("imgly_tool_text",
                                    R.string.pesdk_text_title_name,
                                    ImageSource.create(R.drawable.imgly_icon_tool_text))
                    )
                  }
                  .configure<UiConfigSticker> {
                    val arrLogoSticker = ArrayList<AbstractIdItem>()

                    // Add-logo button
                    arrLogoSticker.add(PersonalStickerAddItem("imgly_add_personal_sticker",
                            R.string.pesdk_sticker_button_add,
                            ImageSource.create(R.drawable.ic_add_logo)))

                    // Stickers & logos
                    if( arrStickers.isNotEmpty() ) {
                      arrLogoSticker.add(StickerCategoryItem(
                              "stickers_id",
                              "stickers",
                              ImageSource.create(arrStickers[0].uri),
                              arrStickers.map { s -> ImageStickerItem.createFromAsset(s.image) }
                      ))
                      arrLogoSticker.add(StickerCategoryItem(
                              "imgly_add_personal_sticker",
                              "logos",
                              ImageSource.create(arrLogos[0].uri),
                              arrLogos.map { s -> ImageStickerItem.createFromAsset(s.image) }
                      ))
                    }
                    /*if( arrLogos.isNotEmpty() ) {
                      arrLogos.map { s ->
                        it.addToPersonalStickerList(ImageStickerItem.createFromAsset(s.image)) }
                    }*/
                    it.setStickerLists(arrLogoSticker)
                  }
                  .configure<AssetConfig> {
                    arrLogos.map { logo -> it.addAsset(logo.image) }
                    arrStickers.map { sticker -> it.addAsset(sticker.image) }
                  }

  val PESDK_RESULT = 1000

  private fun openEditor(inputImage: Uri?) {
    val settingsList = createPesdkSettingsList(inputImage)

    Thread {
      PhotoEditorBuilder(activity.get())
              .setSettingsList(settingsList)
              .startActivityForResult(activity.get(), PESDK_RESULT)
    }.run()
  }

  override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent): Boolean {

    if (resultCode == Activity.RESULT_OK && requestCode == PESDK_RESULT) {
      val settings = data.getParcelableExtra<SettingsList?>(ImgLyIntent.SETTINGS_LIST)
      val model = settings!!.getSettingsModel(AssetConfig::class.java)
      val mapAsset = model.getAssetMap(ImageStickerAsset::class.java)

      val arrLogosSticker = ArrayList<ImageStickerAsset>()
      arrLogos.map { logos -> arrLogosSticker.add(logos.image) }
      arrStickers.map { logos -> arrLogosSticker.add(logos.image) }

      val dynamicStickers = ArrayList<ImageStickerAsset>()
      mapAsset.map { map -> dynamicStickers.add(map) }

      if( dynamicStickers.size > arrLogosSticker.size ) {
        progressDialog = ProgressDialog(activity.get()!!)
        progressDialog.setMessage("Enviando dados...")
        progressDialog.setCancelable(false)
        progressDialog.show()

        val newLogo = ArrayList<ImageStickerAsset>()

        for (new in dynamicStickers) {
          var found = false
          for (logo in arrLogosSticker) {
            if (new.id == logo.id) {
              found = true
              break
            }
          }
          if (!found) newLogo.add(new)
        }

        newLogo.forEachIndexed { i, logo -> Thread {
          uploadLogo(persistLogo(logo.stickerSource.bitmap!!), i == newLogo.size - 1, data)
        }.start()}
      } else {
        savePost(data)
      }
      return true
    }
    return false
  }

  private fun savePost(data: Intent) {
    data.getParcelableExtra<Uri?>(ImgLyIntent.RESULT_IMAGE_URI)?.also {
      activity.get()!!.sendBroadcast(Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE).setData(it))
      val filePath = it.path
      val file = File(filePath)
      val b = file.readBytes()
      resultClass.success(b)
    }
  }

  private fun persistLogo(bitmap: Bitmap, name: String = "temporary_logo"): File {
    val filesDir = activity.get()!!.applicationContext.filesDir
    val imageFile = File(filesDir, "$name.png")

    try {
      val os = FileOutputStream(imageFile)
      bitmap.compress(Bitmap.CompressFormat.PNG, 100, os)
      os.flush()
      os.close()
    } catch (e: Exception) {
      Log.e(this.javaClass.simpleName, "Error writing bitmap", e)
    }
    return imageFile
  }

  private fun uploadLogo(file: File, lastOne: Boolean, data: Intent) {
    val requestBody = RequestBody.create(MediaType.parse("*/*"), file)
    val fileToUpload = MultipartBody.Part.createFormData("logo", file.name, requestBody)

    val getResponse = ApiClient.getClient().create(ApiService::class.java)
    val call = getResponse.addLogo(userId, fileToUpload)
    call.enqueue(object : Callback<LogoToResponse> {
      override fun onFailure(call: Call<LogoToResponse>, t: Throwable) {
        progressDialog.dismiss()
      }

      override fun onResponse(call: Call<LogoToResponse>, response: Response<LogoToResponse>) {
        progressDialog.dismiss()
        if( lastOne ) savePost(data)
      }
    })
  }

  data class CustomImageStickerAsset(val image: ImageStickerAsset, val uri: Uri)
}